@@ -9,4 +9,4 @@ OWLAPI licence: LGPL and Apache
 """
 
 # the import order must be fixed otherwise there are circular import errors
-import owlapy.model
+import owlapy.model  # noqa: F401
